<?php

namespace Drupal\synamo\Service;

use Drupal\Component\Serialization\Json;

/**
 * Class Amo CRM API.
 */
class AmoCRMAPI {

  /**
   * Создаем объект с данными пользователя.
   */
  public function __construct() {
    $config = \Drupal::service('config.factory')->get('synamo.amo_form');
    $this->amoUser = [
      'USER_LOGIN' => $config->get('login'),
      'USER_HASH' => $config->get('hash'),
    ];
    $this->subdomain = $config->get('subdomain');
    $this->cookieDir = DRUPAL_ROOT . '/sites/default/private/amoCookie.txt';

    $this->label = 'Форма с сайта';
    $this->defaultStatus = '9893595';
    $this->defaultTags = 'c сайтa';
    $this->defaultUser = '520377';
  }

  /**
   * Авторизация в амо.
   */
  public function amoAuth() {
    $user = $this->amoUser;
    $link = 'https://' . $this->subdomain . '.amocrm.ru/private/api/auth.php?type=json';

    $curl = curl_init();
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($curl, CURLOPT_USERAGENT, 'amoCRM-API-client/1.0');
    curl_setopt($curl, CURLOPT_URL, $link);
    curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
    curl_setopt($curl, CURLOPT_POSTFIELDS, Json::encode($user));
    curl_setopt($curl, CURLOPT_HTTPHEADER, ['Content-Type: application/json']);
    curl_setopt($curl, CURLOPT_HEADER, FALSE);
    curl_setopt($curl, CURLOPT_COOKIEFILE, $this->cookieDir);
    curl_setopt($curl, CURLOPT_COOKIEJAR, $this->cookieDir);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
    $out = curl_exec($curl);
    $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);

    return Json::decode($out);
  }

  /**
   * получить сделки.
   */
  public function getLeads($from = FALSE, $offset = FALSE, $limit = '500') {
    $args = '';
    if ($offset) {
      $args .= 'limit_rows=' . $limit;
      $args .= '&limit_offset=' . $offset;
    }
    $link = 'https://' . $this->subdomain . '.amocrm.ru/private/api/v2/json/leads/list?' . $args;

    $curl = curl_init();
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($curl, CURLOPT_USERAGENT, 'amoCRM-API-client/1.0');
    curl_setopt($curl, CURLOPT_HEADER, FALSE);
    curl_setopt($curl, CURLOPT_COOKIEFILE, $this->cookieDir);
    curl_setopt($curl, CURLOPT_COOKIEJAR, $this->cookieDir);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($curl, CURLOPT_URL, $link);
    if ($from) {
      curl_setopt($curl, CURLOPT_HTTPHEADER, [
        'IF-MODIFIED-SINCE: ' . \Drupal::service('date.formatter')->format($from, 'custom', 'D, d M Y H:i:s', NULL, 'en'),
      ]);
    }
    $out = curl_exec($curl);
    $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);

    return Json::decode($out);
  }

  /**
   * Инфо по аккаунту.
   */
  public function currentAccountInfo() {
    $link = 'https://' . $this->subdomain . '.amocrm.ru/private/api/v2/json/accounts/current';
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($curl, CURLOPT_USERAGENT, 'amoCRM-API-client/1.0');
    curl_setopt($curl, CURLOPT_URL, $link);
    curl_setopt($curl, CURLOPT_HEADER, FALSE);
    curl_setopt($curl, CURLOPT_COOKIEFILE, $this->cookieDir);
    curl_setopt($curl, CURLOPT_COOKIEJAR, $this->cookieDir);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
    $out = curl_exec($curl);
    $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);
    return Json::decode($out);
  }

  /**
   * Добавить сделку.
   */
  public function addLead($lead) {
    $leads['request']['leads']['add'] = [$lead];
    $link = 'https://' . $this->subdomain . '.amocrm.ru/private/api/v2/json/leads/set';
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($curl, CURLOPT_USERAGENT, 'amoCRM-API-client/1.0');
    curl_setopt($curl, CURLOPT_URL, $link);
    curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
    curl_setopt($curl, CURLOPT_POSTFIELDS, Json::encode($leads));
    curl_setopt($curl, CURLOPT_HTTPHEADER, ['Content-Type: application/json']);
    curl_setopt($curl, CURLOPT_HEADER, FALSE);
    curl_setopt($curl, CURLOPT_COOKIEFILE, $this->cookieDir);
    curl_setopt($curl, CURLOPT_COOKIEJAR, $this->cookieDir);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
    $out = curl_exec($curl);
    $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);
    return Json::decode($out);
  }

  /**
   * Custom field.
   */
  private function constructCustomField($field) {
    $result = FALSE;
    if ($field['id']) {
      $result = [
        'id' => $field['id'],
        'values' => [
          [
            'value' => $field['value'],
          ],
        ],
      ];
    }
    return $result;
  }

  /**
   * Добавить контакт.
   */
  public function addContact($contact) {
    $contacts['request']['contacts']['add'] = [$contact];
    $link = 'https://' . $this->subdomain . '.amocrm.ru/private/api/v2/json/contacts/set';
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($curl, CURLOPT_USERAGENT, 'amoCRM-API-client/1.0');
    curl_setopt($curl, CURLOPT_URL, $link);
    curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
    curl_setopt($curl, CURLOPT_POSTFIELDS, Json::encode($contacts));
    curl_setopt($curl, CURLOPT_HTTPHEADER, ['Content-Type: application/json']);
    curl_setopt($curl, CURLOPT_HEADER, FALSE);
    curl_setopt($curl, CURLOPT_COOKIEFILE, $this->cookieDir);
    curl_setopt($curl, CURLOPT_COOKIEJAR, $this->cookieDir);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
    $out = curl_exec($curl);
    $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);
    return Json::decode($out);
  }

  /**
   * Добавить событие.
   */
  public function addNotes($notes) {
    $notes['request']['notes']['add'] = [$notes];
    $link = 'https://' . $this->subdomain . '.amocrm.ru/private/api/v2/json/notes/set';
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($curl, CURLOPT_USERAGENT, 'amoCRM-API-client/1.0');
    curl_setopt($curl, CURLOPT_URL, $link);
    curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
    curl_setopt($curl, CURLOPT_POSTFIELDS, Json::encode($notes));
    curl_setopt($curl, CURLOPT_HTTPHEADER, ['Content-Type: application/json']);
    curl_setopt($curl, CURLOPT_HEADER, FALSE);
    curl_setopt($curl, CURLOPT_COOKIEFILE, $this->cookieDir);
    curl_setopt($curl, CURLOPT_COOKIEJAR, $this->cookieDir);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
    $out = curl_exec($curl);
    $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);
    return Json::decode($out);
  }

  /**
   * Добавить компанию.
   */
  public function addCompany($options) {
    $company = [];
    $company['name'] = $options['name'];
    $company['responsible_user_id'] = $options['responsible'];
    $company['tags'] = $options['tags'];
    $customFields = [];

    if (isset($options['vkField'])) {
      if ($field = $this->constructCustomField($options['vkField'])) {
        $customFields[] = $field;
      }
    }

    if (isset($options['descrField'])) {
      if ($field = $this->constructCustomField($options['descrField'])) {
        $customFields[] = $field;
      }
    }

    if (isset($options['vkLink'])) {
      if ($field = $this->constructCustomField($options['vkLink'])) {
        $customFields[] = $field;
      }
    }

    if (count($customFields)) {
      $company['custom_fields'] = $customFields;
    }

    $contacts['request']['contacts']['add'] = [$company];

    $link = 'https://' . $this->subdomain . '.amocrm.ru/private/api/v2/json/company/set';

    $curl = curl_init();
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($curl, CURLOPT_USERAGENT, 'amoCRM-API-client/1.0');
    curl_setopt($curl, CURLOPT_URL, $link);
    curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
    curl_setopt($curl, CURLOPT_POSTFIELDS, Json::encode($contacts));
    curl_setopt($curl, CURLOPT_HTTPHEADER, ['Content-Type: application/json']);
    curl_setopt($curl, CURLOPT_HEADER, FALSE);
    curl_setopt($curl, CURLOPT_COOKIEFILE, $this->cookieDir);
    curl_setopt($curl, CURLOPT_COOKIEJAR, $this->cookieDir);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
    $out = curl_exec($curl);
    $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);

    return Json::decode($out);
  }

  /**
   * Найти поле.
   */
  public function findField($customFields, $field) {
    $fieldId = FALSE;
    foreach ($customFields as $row) {
      if (strripos($row['name'], $field) !== FALSE) {
        $fieldId = $row['id'];
      }
    }
    return $fieldId;
  }

  /**
   * Найти компанию.
   */
  public function findCompany($options = []) {
    $link = 'https://' . $this->subdomain . '.amocrm.ru/private/api/v2/json/company/list?' . http_build_query($options);
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($curl, CURLOPT_USERAGENT, 'amoCRM-API-client/1.0');
    curl_setopt($curl, CURLOPT_HEADER, FALSE);
    curl_setopt($curl, CURLOPT_COOKIEFILE, $this->cookieDir);
    curl_setopt($curl, CURLOPT_COOKIEJAR, $this->cookieDir);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($curl, CURLOPT_URL, $link);
    $out = curl_exec($curl);
    $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);

    return Json::decode($out);
  }

  /**
   * Найти контакт.
   */
  public function findContactEmail($email) {
    $link = 'https://' . $this->subdomain . '.amocrm.ru/private/api/v2/json/contacts/list?query=' . $email;
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($curl, CURLOPT_USERAGENT, 'amoCRM-API-client/1.0');
    curl_setopt($curl, CURLOPT_HEADER, FALSE);
    curl_setopt($curl, CURLOPT_COOKIEFILE, $this->cookieDir);
    curl_setopt($curl, CURLOPT_COOKIEJAR, $this->cookieDir);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($curl, CURLOPT_URL, $link);
    $out = curl_exec($curl);
    $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);

    return Json::decode($out);
  }

  /**
   * Найти контакт.
   */
  public function findContact($options = []) {
    $sub = $this->subdomain;
    $query = http_build_query($options);
    $link = "https://$sub.amocrm.ru/private/api/v2/json/contacts/?{$query}";
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($curl, CURLOPT_USERAGENT, 'amoCRM-API-client/1.0');
    curl_setopt($curl, CURLOPT_HEADER, FALSE);
    curl_setopt($curl, CURLOPT_COOKIEFILE, $this->cookieDir);
    curl_setopt($curl, CURLOPT_COOKIEJAR, $this->cookieDir);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($curl, CURLOPT_URL, $link);
    $out = curl_exec($curl);
    $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);

    return Json::decode($out);
  }

  /**
   * Обновить.
   */
  public function updateCompany($options = []) {
    $link = 'https://' . $this->subdomain . '.amocrm.ru/private/api/v2/json/contacts/update?' . http_build_query($options);
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($curl, CURLOPT_USERAGENT, 'amoCRM-API-client/1.0');
    curl_setopt($curl, CURLOPT_HEADER, FALSE);
    curl_setopt($curl, CURLOPT_COOKIEFILE, $this->cookieDir);
    curl_setopt($curl, CURLOPT_COOKIEJAR, $this->cookieDir);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($curl, CURLOPT_URL, $link);
    $out = curl_exec($curl);
    $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);

    return Json::decode($out);
  }

  /**
   * Воронки.
   */
  public function getPipelines($from = FALSE, $offset = FALSE, $limit = '500') {
    $args = '';
    if ($offset) {
      $args .= 'limit_rows=' . $limit;
      $args .= '&limit_offset=' . $offset;
    }
    $link = 'https://' . $this->subdomain . '.amocrm.ru/private/api/v2/json/pipelines/list?' . $args;

    $curl = curl_init();
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($curl, CURLOPT_USERAGENT, 'amoCRM-API-client/1.0');
    curl_setopt($curl, CURLOPT_HEADER, FALSE);
    curl_setopt($curl, CURLOPT_COOKIEFILE, $this->cookieDir);
    curl_setopt($curl, CURLOPT_COOKIEJAR, $this->cookieDir);
    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($curl, CURLOPT_URL, $link);
    if ($from) {
      curl_setopt($curl, CURLOPT_HTTPHEADER, [
        'IF-MODIFIED-SINCE: ' . \Drupal::service('date.formatter')->format($from, 'custom', 'D, d M Y H:i:s', NULL, 'en'),
      ]);
    }
    $out = curl_exec($curl);
    $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);

    return Json::decode($out);
  }

}
